@@ -94,10 +94,11 @@ p4dti_en_catalog = {
           "P4DTI configuration specifies Bugzilla version %s, but version %s detected."),
     127: (message.NOT_USED,
           "Bugzilla's fielddefs table does not include '%s'."),
-    128: (message.INFO, "Running processmail for %d bugs..."),
+    128: (message.INFO, "Running %d deferred commands..."),
     129: (message.WARNING, "The Bugzilla configuration parameters are missing from the Bugzilla database.  This means that the P4DTI won't support Bugzilla features like 'emailsuffix'.  If you need these features, edit your Bugzilla configuration parameters and restart the P4DTI.  See section 5.3.3 of the P4DTI Administrator's Guide."),
     130: (message.WARNING, "Bugzilla configuration parameter 'p4dti' is turned off.  You won't see Perforce fixes in Bugzilla until you turn it on.  See section 5.3.3 of the P4DTI Administrator's Guide."),
-
+    131: (message.WARNING, "Bugzilla version %s detected, with these tables missing: %s and these additional tables present: %s. The P4DTI may fail to operate correctly."),
+    132: (message.WARNING, "Bugzilla version %s detected, with these tables missing: %s.  The P4DTI may fail to operate correctly."),
 
     # 2.2. Messages from check_config.py (200-299)
 
@@ -113,6 +114,7 @@ p4dti_en_catalog = {
     209: (message.CRIT, "Configuration parameter '%s' (value '%s') must be from 1 to 32 characters long, start with a letter or number, and consist of letters, numbers and underscores only."),
     210: (message.CRIT, "Configuration parameter '%s' (value '%s') must contain exactly one %%d format specifier, any number of doubled percents, but no other format specifiers."),
     211: (message.CRIT, "Configuration parameter '%s' (value '%s') must contain exactly one %%s format specifier, any number of doubled percents, but no other format specifiers."),
+    212: (message.CRIT, "Configuration parameter '%s' must be a list of pairs of strings."),
 
 
     # 2.3. Messages from configure_bugzilla.py (300-399)
@@ -121,7 +123,7 @@ p4dti_en_catalog = {
     301: (message.CRIT, "You specified the closed_state '%s', but there's no such Bugzilla state."),
     302: (message.CRIT, "The '%s' column of Bugzilla's 'bugs' table is not an enum type."),
     303: (message.CRIT, "Configuration parameter 'bugzilla_directory' does not name a directory."),
-    304: (message.CRIT, "Configuration parameter 'bugzilla_directory' does not name a directory containing a processmail script."),
+    304: (message.CRIT, "Configuration parameter 'bugzilla_directory' does not name a directory containing a mail-processing script."),
     305: (message.CRIT, "Bugzilla's table 'profiles' does not have a 'login_name' column."),
     306: (message.CRIT, "The 'login_name' column of Bugzilla's 'profiles' table does not have a 'text' type."),
     307: (message.CRIT, "Bugzilla's table 'bugs' does not have a '%s' column."),
@@ -130,12 +132,18 @@ p4dti_en_catalog = {
     310: (message.CRIT, "The 'resolution' column of Bugzilla's 'bugs' table does not have a 'FIXED' value."),
     311: (message.CRIT, "Field '%s' specified in 'replicated_fields' is a system field: leave it out!"),
     312: (message.CRIT, "Field '%s' appears twice in 'replicated_fields'."),
-    313: (message.CRIT, "Field '%s' specified in 'replicated_fields' list not in Bugzilla 'bugs' table."),
+    313: (message.NOT_USED, "Field '%s' specified in 'replicated_fields' list not in Bugzilla 'bugs' table."),
     314: (message.CRIT, "Field '%s' specified in 'replicated_fields' list has type '%s': this is not yet supported by P4DTI."),
     315: (message.CRIT, "Field '%s' specified in 'replicated_fields' list has floating-point type: this is not yet supported by P4DTI."),
     316: (message.CRIT, "You can't have a field called 'code' in the Perforce jobspec."),
     317: (message.CRIT, "Too many fields to replicate: Perforce jobs can contain only 99 fields."),
     318: (message.NOT_USED, "Jobspec fields '%s' and '%s' have the same number %d."),
+    319: (message.CRIT, "Field '%s' specified in 'omitted_fields' is not a system field: leave it out!"),
+    320: (message.CRIT, "Field '%s' appears twice in 'omitted_fields'."),
+    321: (message.CRIT, "Field '%s' in 'field_names' is not a replicated field."),
+    322: (message.CRIT, "Bugzilla field '%s' appears twice in 'field_names'."),
+    323: (message.CRIT, "Perforce field '%s' appears twice in 'field_names'."),
+    324: (message.CRIT, "Bugzilla fields '%s' and '%s' both map to Perforce field '%s'."),
 
 
     # 2.4. Messages from configure_teamtrack.py (400-499)
@@ -154,14 +162,14 @@ p4dti_en_catalog = {
 
     # 2.5. Messages from dt_bugzilla.py (500-599)
 
-    500: (message.ERR, "User %d isn't in the right bug group to edit bug %d."),
+    500: (message.NOT_USED, "User %d isn't in the right bug group to edit bug %d."),
     501: (message.ERR, "User %d doesn't have permission to change field '%s' of bug %d to %s."),
     502: (message.ERR, "The P4DTI does not support marking bugs as DUPLICATE from Perforce."),
     503: (message.ERR, "Bugzilla does not allow a transition from status '%s' to '%s'."),
     504: (message.ERR, "Cannot change Bugzilla field '%s'."),
     505: (message.ERR, "Can only append to Bugzilla field '%s'."),
     506: (message.ERR, "Updating non-existent Bugzilla field '%s'."),
-    507: (message.ERR, "Bugzilla does not have a group called '%s'."),
+    507: (message.NOT_USED, "Bugzilla does not have a group called '%s'."),
     508: (message.NOT_USED, "Bugzilla's fielddefs table does not include '%s'."),
     509: (message.ERR, "No Perforce status corresponding to Bugzilla status '%s'."),
     510: (message.ERR, "No Bugzilla status corresponding to Perforce status '%s'."),
@@ -181,9 +189,9 @@ p4dti_en_catalog = {
     524: (message.ERR, "Can't create Bugzilla bug for product '%s' with no versions."),
     525: (message.ERR, "Can't create Bugzilla bug without version field."),
     526: (message.ERR,  "Can't create Bugzilla bug: product '%s' has no version '%s'."),
-    527: (message.NOTICE, "User '%s' isn't in Bugzilla product group for product '%s'; migrating bug anyway."),
-    528: (message.ERR, "Can't create Bugzilla bug with invalid groupset '%s'."),
-    529: (message.NOTICE, "User '%s' doesn't have permissions to create Bugzilla bug for product '%s' with status '%s'; migrating bug anyway."),
+    527: (message.NOTICE, "User '%s' isn't in Bugzilla group '%s' required to make bug for product '%s'; creating bug anyway."),
+    528: (message.ERR, "Can't create Bugzilla bug with invalid group '%s'."),
+    529: (message.NOTICE, "User '%s' doesn't have permissions to create Bugzilla bug for product '%s' with status '%s'; creating bug anyway."),
     530: (message.ERR, "Can't create Bugzilla bug with bug_status '%s' and no resolution."),
     531: (message.ERR, "Can't create Bugzilla bug with field '%s'."),
     532: (message.ERR, "Can't create Bugzilla bug without reporter field."),
@@ -207,8 +215,10 @@ p4dti_en_catalog = {
     550: (message.ERR, "Bugzilla P4DTI user '%s' does not have a matching Perforce user.  It should match the Perforce user '%s' but that matches the Bugzilla user %d (e-mail address '%s')."),
     551: (message.ERR, "Bugzilla P4DTI user '%s' does not have a matching Perforce user.  It should match the Perforce user '%s' (which has e-mail address '%s')."),
     552: (message.NOTICE, "These Bugzilla users have duplicate e-mail addresses (when converted to lower case).  They may have been matched with the wrong Perforce user."),
-    553:  (message.NOTICE, "Perforce replicator user <%s> already exists in Bugzilla as user %d."),
-    554:  (message.NOTICE, "Perforce replicator user <%s> added to Bugzilla as user %d."),
+    553: (message.NOTICE, "Perforce replicator user <%s> already exists in Bugzilla as user %d."),
+    554: (message.NOTICE, "Perforce replicator user <%s> added to Bugzilla as user %d."),
+    555: (message.ERR, "User %d must be in group '%s' to edit bug %d."),
+    556: (message.ERR, "User %d must be in group '%s' to edit bug %d in product '%s'."),
 
     # 2.6. Messages from dt_teamtrack.py (600-699)
     # That module has been removed, so all these messages are now NOT_USED.
@@ -267,12 +277,36 @@ p4dti_en_catalog = {
     704: (message.ERR, "Perforce client changelevel %d is not supported by P4DTI.  Client must be at changelevel %d or above."),
     705: (message.ERR, "The command '%s' didn't report a recognizable version number.  Check your setting for the 'p4_client_executable' parameter."),
     706: (message.ERR, "%s  The Perforce client exited with error code %d."),
-    707: (message.ERR, "The Perforce client exited with error code %d.  The server might be down; the server address might be incorrect; or your Perforce licence might have expired."),
+    707: (message.ERR, "The Perforce client exited with error code %d.  The server might be down; the server address might be incorrect; or your Perforce license might have expired."),
     708: (message.ERR, "%s"),
-    709: (message.CRIT, "The Perforce interface does not support the operating system '%s'."),
+    709: (message.NOT_USED, "The Perforce interface does not support the operating system '%s'."),
     710: (message.CRIT, "Jobspec fields '%s' and '%s' have the same number %d."),
     711: (message.DEBUG, "Decoded jobspec as comment '%s' and fields %s."),
     712: (message.DEBUG, "Installing jobspec from comment '%s' and fields %s."),
+    713: (message.WARNING, "Jobspec does not have required P4DTI field '%s'."),
+    714: (message.WARNING, "Jobspec P4DTI field '%s' has incorrect attribute '%s': '%s' (should be '%s')."),
+    715: (message.ERR, "Jobspec does not support P4DTI."),
+    716: (message.WARNING, "Jobspec does not have field '%s'."),
+    717: (message.ERR, "Current jobspec cannot be used for replication."),
+    718: (message.WARNING, "The jobspec does not allow values '%s' in field '%s', so these values cannot be replicated from the defect tracker."),
+    719: (message.WARNING, "The jobspec does not allow value '%s' in field '%s', so this value cannot be replicated from the defect tracker."),
+    720: (message.WARNING, "Field '%s' in the jobspec allows values '%s', which cannot be replicated to the defect tracker."),
+    721: (message.WARNING, "Field '%s' in the jobspec allows value '%s', which cannot be replicated to the defect tracker."),
+    722: (message.WARNING, "Jobspec field '%s' has a more restrictive datatype ('%s' not '%s') which may cause problems replicating this field from the defect tracker."),
+    723: (message.WARNING, "Jobspec field '%s' has a less restrictive datatype ('%s' not '%s') which may cause problems replicating this field to the defect tracker."),
+    724: (message.WARNING, "Field '%s' in the jobspec should be a '%s' field, not '%s'.  This field cannot be replicated to or from the defect tracker."),
+    725: (message.WARNING, "Field '%s' in the jobspec should have persistence '%s', not '%s'.  There may be problems replicating this field to or from the defect tracker."),
+    726: (message.WARNING, "Perforce job field '%s' will not be replicated to the defect tracker."),
+    727: (message.WARNING, "Forcing replacement of field '%s' in jobspec."),
+    728: (message.INFO, "Retaining field '%s' in jobspec despite change."),
+    729: (message.INFO, "Adding field '%s' to jobspec."),
+    730: (message.ERR, "Too many fields in jobspec."),
+    731: (message.WARNING, "Jobspec field '%s' has unknown datatype '%s' which may cause problems when replicating this field."),
+    732: (message.INFO, "Retaining unknown field '%s' in jobspec."),
+    733: (message.INFO, "No change to field '%s' in jobspec."),
+    734: (message.INFO, "Perforce message '%s'.  Switching Unicode mode %s to retry."),
+    735: (message.ERR,  "Perforce message '%s'.  Reverting to Unicode mode %s."),
+
 
 
     # 2.8. Messages from replicator.py (800-999)
@@ -311,7 +345,7 @@ p4dti_en_catalog = {
     833: (message.CRIT, "The replicator's RID ('%s') doesn't match the defect tracker's RID ('%s')."),
     834: (message.CRIT, "The Perforce server changelevel %d is not supported by the P4DTI.  See the P4DTI release notes for Perforce server versions supported by the P4DTI."),
     835: (message.CRIT, "The Perforce command 'p4 info' didn't report a recognisable version."),
-    836: (message.CRIT, "P4DTI fields not found in Perforce jobspec."),
+    836: (message.NOT_USED, "P4DTI fields not found in Perforce jobspec."),
     837: (message.ERR, "Expected a job but found %s."),
     838: (message.ERR, "Asked for job '%s' but got job '%s'."),
     839: (message.ERR, "P4DTI-filespecs field has value '%s': this should end in a newline."),
@@ -405,7 +439,7 @@ p4dti_en_catalog = {
     928: (message.WARNING, "Attempting to make working Perforce client %s."),
 
     # 2.9. Messages from init.py, check.py, check_jobs.py, run.py,
-    # refresh.py, mysqldb_support.py, service.py,
+    # refresh.py, mysqldb_support.py, service.py, portable.py,
     # logger.py (1000-1099)
 
     1000: (message.NOT_USED, "The defect tracker '%s' is not supported."),
@@ -429,6 +463,9 @@ p4dti_en_catalog = {
     1018: (message.ERR, "An attempt to write a log message to %s failed."),
     1019: (message.ERR, "An attempt to write a log message to the system log failed."),
     1020: (message.ERR, "An attempt to write a log message to the NT event log failed."),
+    1021: (message.CRIT, "The P4DTI does not support the operating system '%s'."),
+    1022: (message.WARNING, "MySQLdb version '%s' (release '%s') detected.  This release is supported by the P4DTI, but deprecated.  Future versions of the P4DTI may not support this release."),
+    1023: (message.WARNING, "MySQLdb version '%s' (release '%s') detected. This old release is not supported by the P4DTI, and may not provide functions on which the P4DTI relies."),
 
     # 2.10. Messages from teamtrack_query.py (1100-1199)
     # That module has been removed, so all these messages are now NOT_USED.
@@ -456,7 +493,7 @@ def msg(id, args = ()):
 #
 # [GDR 2000-10-16] "Perforce Defect Tracking Integration Integrator's
 # Guide"; Richard Brooksby; Ravenbrook Limited; 2000-10-16;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/manual/ig/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/manual/ig/>.
 #
 # [ISO 639] "Code for the representation of names of languages"; ISO;
 # 1988-04-01.
@@ -561,8 +598,23 @@ def msg(id, args = ()):
 #
 # 2003-05-30 NB Add 927, 928.
 #
+# 2003-09-17 NB Moved 709 to 1021.
 #
-# C. COPYRIGHT AND LICENCE
+# 2003-09-24 NB Added 131 and 132.
+#
+# 2003-11-25 NB Added 212.
+#
+# 2003-12-05 NB Added 713, 714, 715.
+#
+# 2003-12-10 NB Add 716 to 726.
+#
+# 2003-12-12 NB Add 727 to 733; make 836 NOT_USED.
+#
+# 2004-05-28 NB Bugzilla 2.17.7 support (added 555, 556; removed 500,
+# 507; changed 128, 304, 527-529).
+#
+#
+# C. COPYRIGHT AND LICENSE
 #
 # This file is copyright (c) 2001 Perforce Software, Inc.  All rights
 # reserved.
@@ -593,4 +645,4 @@ def msg(id, args = ()):
 # DAMAGE.
 #
 #
-# $Id: //info.ravenbrook.com/project/p4dti/version/2.0/code/replicator/catalog.py#2 $
+# $Id: //info.ravenbrook.com/project/p4dti/version/2.3/code/replicator/catalog.py#3 $
