@@ -47,7 +47,7 @@ enum_translator = dt_bugzilla.enum_translator(keyword_translator)
 #
 # The closed_state argument is the Bugzilla state which maps to the
 # special state 'closed' in Perforce, or None if there is no such state.
-# See requirement 45.  See decision decision [RB 2000-11-28 14:44:36
+# See requirement 45.  See design decision [RB 2000-11-28 14:44:36
 # GMT].
 #
 # The case of state names in these pairs is normalized for usability in
@@ -110,15 +110,14 @@ def make_state_pairs(states, closed_state):
 
 
 # Given the name of an enum column, calculate the values and default.
-def translate_enum(bugs_types, column):
-    if not bugs_types[column]['type'] == 'enum':
+def translate_enum(column, bz_type):
+    if not bz_type['type'] == 'enum':
         # "The '%s' column of Bugzilla's 'bugs' table is not an enum
         # type."
         raise error, catalog.msg(302, column)
-    values = bugs_types[column]['values']
-    values = map(enum_translator.translate_0_to_1,
-                 bugs_types[column]['values'])
-    default = bugs_types[column]['default']
+    values = bz_type['values']
+    values = map(enum_translator.translate_0_to_1, values)
+    default = bz_type['default']
     if default != None:
         default = enum_translator.translate_0_to_1(default)
     values = string.join(values,'/')
@@ -144,7 +143,10 @@ read_only_fields = ['bug_id',
                     'votes',
                     'keywords',
                     'lastdiffed',
-                    'everconfirmed']
+                    'everconfirmed',
+                    'estimated_time',
+                    'remaining_time',
+                    'alias']
 
 # Some Bugzilla fields can only be appended to.
 # In particular, the 'Descriptions' section of a Bugzilla bug
@@ -155,6 +157,15 @@ read_only_fields = ['bug_id',
 
 append_only_fields = ['longdesc']
 
+# These Bugzilla fields are replicated by default.  If they are in
+# replicated_fields, that's a configuration error.  If fields other
+# than these are in omitted_fields, that's a configuration error too.
+default_fields = ['bug_status',
+                  'assigned_to',
+                  'short_desc',
+                  'resolution',
+                  ]
+
 # Map bugzilla field name to a name we can use in Perforce and a
 # comment.
 
@@ -163,7 +174,8 @@ bz_field_map = {
                           "Description and comments."),
     'assigned_to':       ('Assigned_To',
                           "User to which the bug is assigned."),
-    'groupset':          ('Groupset', None),
+    'groupset':          ('Groupset',
+                          None),
     'bug_file_loc':      ('URL',
                           "The bug's URL."),
     'bug_severity':      ('Severity',
@@ -218,13 +230,21 @@ bz_field_map = {
                              "Accessible to the QA Contact."),
     'cclist_accessible': ('CCList_accessible',
                           "Accessible to the CC List."),
+    'alias':             ('Alias',
+                          "Bug alias."),
+    'estimated_time':    ('EstimatedTime',
+                          "Estimated time in hours."),
+    'remaining_time':    ('RemainingTime',
+                          "Remaining time in hours."),
+    'deadline':          ('Deadline',
+                          "Deadline for fixing the bug."),
     }
 
 
 def prepare_issue_advanced(config, bz, p4, dict, job):
     # Deduce a reporter for the issue, unless we have one already.
     if dict.get('reporter', "") == "":
-        for field in ['P4DTI-user', 'Assigned_To', 'User']:
+        for field in ['P4DTI-user', config.job_owner_field, 'User']:
             if job.has_key(field):
                 p4_user = job[field]
                 # note: this is a lax translator
@@ -236,9 +256,9 @@ def prepare_issue_advanced(config, bz, p4, dict, job):
 
     # Set creation_ts to the 'Date' field, suitably translated.
     # (otherwise creation_ts gets now()).
-    if job.has_key('Date') and dict.get('creation_ts', '') == '':
+    if job.has_key(config.job_date_field) and dict.get('creation_ts', '') == '':
         dict['creation_ts'] = config.date_translator.translate_1_to_0(
-            job['Date'], bz, p4)
+            job[config.job_date_field], bz, p4)
 
     # If no summary, get short description from the first line of the
     # long description.
@@ -259,7 +279,7 @@ def prepare_issue_advanced(config, bz, p4, dict, job):
     if (dict.has_key('bug_status')
         and dict['bug_status'] in bugzilla_resolved_statuses
         and dict.get('resolution','') == ''):
-        if job.get('Status', '') == 'suspended':
+        if job.get(config.job_status_field, '') == 'suspended':
             dict['resolution'] = 'LATER'
         else:
             dict['resolution'] = 'FIXED'
@@ -277,6 +297,220 @@ def translate_jobspec_advanced(config, dt, p4, job):
     # Call user hook (see [GDR 2001-11-14, 4.6]).
     return config.translate_jobspec(job)
 
+def get_fields_bz_to_p4(config, bugs_types):
+    fields_bz_to_p4 = {}
+    fields_p4_to_bz = {}
+    # particular requested mappings
+    field_names_bz_to_p4 = {}
+
+    bz_fields = default_fields + config.replicated_fields
+    for f in config.omitted_fields:
+        bz_fields.remove(f)
+    
+    for (bz_field, p4_field) in config.field_names:
+        field_names_bz_to_p4[bz_field] = p4_field
+    
+    for bz_field in bz_fields:
+        if not bugs_types.has_key(bz_field):
+            # "Bugzilla's table 'bugs' does not have a '%s' column."
+            raise error, catalog.msg(307, bz_field)
+
+        if field_names_bz_to_p4.has_key(bz_field):
+            p4_field = field_names_bz_to_p4[bz_field]
+        elif bz_field_map.has_key(bz_field):
+            p4_field = bz_field_map[bz_field][0]
+        else:
+            p4_field = "bugzilla_" + bz_field
+
+        if fields_p4_to_bz.has_key(p4_field):
+            # "Bugzilla fields '%s' and '%s' both map to Perforce field '%s'."
+            raise error, catalog.msg(324, (bz_field,
+                                           fields_p4_to_bz[p4_field],
+                                           p4_field))
+        fields_bz_to_p4[bz_field] = p4_field
+        fields_p4_to_bz[p4_field] = bz_field
+    return fields_bz_to_p4
+
+
+def check_bugzilla_directory(config):
+    dir = config.bugzilla_directory
+    if dir is None:
+        config.bugmail_command = None
+        return
+    # strip any trailing / character
+    if (len(dir) > 1
+        and (dir[-1:] == '/' or
+             dir[-1:] == '\\')):
+        dir = dir[:-1]
+    if not os.path.isdir(dir):
+        # "Configuration parameter 'bugzilla_directory' does not
+        # name a directory."
+        raise error, catalog.msg(303)
+    config.bugzilla_directory = dir
+    # Check processmail
+    if os.name == 'posix':
+        processmail = 'processmail'
+    elif os.name == 'nt':
+        # processmail name is different on Windows
+        processmail = 'processmail.pl'
+    if os.access(os.path.join(dir, processmail), os.X_OK):
+        config.bugmail_command = processmail
+    else:
+        bugmail = os.path.join('contrib', 'sendbugmail.pl')
+        if os.access(os.path.join(dir, bugmail), os.R_OK):
+            config.bugmail_command = bugmail
+        else:
+            # "Configuration parameter 'bugzilla_directory' does not
+            # name a directory containing a mail-processing script."
+            raise error, catalog.msg(304)
+
+def check_field_lists(config):
+    # 1. Validate the replicated_fields config parameter.
+    bz_fields = []
+    for f in config.replicated_fields:
+        if f in default_fields:
+            # "Field '%s' specified in 'replicated_fields' is a system field: leave it out!"
+            raise error, catalog.msg(311, f)
+        if f in bz_fields:
+            # "Field '%s' appears twice in 'replicated_fields'."
+            raise error, catalog.msg(312, f)
+        else:
+            bz_fields.append(f)
+
+    # 2. Build a list of the fields we will replicate.
+    bz_fields = bz_fields + default_fields
+
+    # 3. Validate the omitted_fields config parameter.
+    fields = []
+    for f in config.omitted_fields:
+        if f not in default_fields:
+            # "Field '%s' specified in 'omitted_fields' is not a system field: leave it out!"
+            raise error, catalog.msg(319, f)
+        if f in fields:
+            # "Field '%s' appears twice in 'omitted_fields'."
+            raise error, catalog.msg(320, f)
+        else:
+            fields.append(f)
+        bz_fields.remove(f)
+
+    # bz_fields is now the set of fields we will replicate.
+
+    # 4. Validate the field_names config parameter.
+    bz_fields_check = []
+    p4_fields_check = []
+    for (bz_field, p4_field) in config.field_names:
+        if bz_field in bz_fields_check:
+            # "Bugzilla field '%s' appears twice in 'field_names'."
+            raise error, catalog.msg(322, bz_field)
+        bz_fields_check.append(bz_field)
+        if bz_field not in bz_fields:
+            # "Field '%s' in 'field_names' is not a replicated field."
+            raise error, catalog.msg(321, bz_field)
+        if p4_field in p4_fields_check:
+            # "Perforce field '%s' appears twice in 'field_names'."
+            raise error, catalog.msg(323, p4_field)
+        p4_fields_check.append(p4_field)
+
+def get_user_name_length(config):
+    # Get the types of the 'profiles' table from Bugzilla.  In
+    # particular we need to know the size of the 'login_name' column.
+    profiles_types = config.bugzilla.get_types('profiles')
+    if not profiles_types.has_key('login_name'):
+        # "Bugzilla's table 'profiles' does not have a 'login_name'
+        # column."
+        raise error, catalog.msg(305)
+    if profiles_types['login_name']['type'] != 'text':
+        # "The 'login_name' column of Bugzilla's 'profiles' table does
+        # not have a 'text' type."
+        raise error, catalog.msg(306)
+    return profiles_types['login_name']['length']
+
+def make_p4_field_spec(bz_field,
+                       p4_field,
+                       bz_type,
+                       user_name_length,
+                       strict_user_translator):
+    if bz_field_map.has_key(bz_field):
+        p4_comment = bz_field_map[bz_field][1]
+    else:
+        p4_comment = None
+        
+    if p4_comment is None:
+        p4_comment = ("Bugzilla's '%s' field" % bz_field)
+        
+    bz_type_class = bz_type['type']
+    p4_values = None
+    p4_length = None
+
+    # if there is a default, use it.
+    if bz_type.get('default'):
+        p4_class = 'default'
+        p4_default = bz_type['default']
+    else:
+        p4_class = 'optional'
+        p4_default = None
+
+    # Figure out the Perforce types, lengths, values, and default, and
+    # the translator.
+    # Maybe this should be table-driven.
+    if bz_type_class == 'float':
+        # "Field '%s' specified in 'replicated_fields' list has
+        # floating-point type: this is not yet supported by P4DTI."
+        raise error, catalog.msg(315, bz_field)
+    elif bz_type_class == 'user':
+        p4_type = 'word'
+        p4_length = user_name_length
+        trans = strict_user_translator
+    elif bz_type_class == 'enum':
+        p4_type = 'select'
+        p4_values, p4_default = translate_enum(bz_field, bz_type)
+        trans = enum_translator
+    elif bz_type_class == 'int':
+        p4_type = 'word'
+        trans = dt_bugzilla.int_translator()
+    elif bz_type_class == 'date':
+        p4_type = 'date'
+        p4_length = 20
+        trans = dt_bugzilla.date_translator()
+    elif bz_type_class == 'timestamp':
+        p4_type = 'date'
+        p4_length = 20
+        trans = dt_bugzilla.timestamp_translator()
+    elif bz_type_class == 'text':
+        p4_type = 'text'
+        trans = dt_bugzilla.text_translator()
+    else:
+        # "Field '%s' specified in 'replicated_fields' list has type
+        # '%s': this is not yet supported by P4DTI."
+        raise error, catalog.msg(314, (bz_field,
+                                       bz_type['sql_type']))
+
+    # "p4 -G" uses the field "code" to indicate whether the Perforce
+    # command succeeded or failed.  See job000003.
+    if p4_field == 'code':
+        # "You can't have a field called 'code' in the Perforce
+        # jobspec."
+        raise error, catalog.msg(316)
+
+    # Fixed-length fields get the length from Bugzilla.
+    if p4_length == None:
+        p4_length = bz_type['length']
+
+    if bz_field in read_only_fields:
+        p4_comment = (p4_comment + "  DO NOT MODIFY.")
+
+    if bz_field in append_only_fields:
+        p4_comment = (p4_comment + "  ONLY MODIFY BY APPENDING.")
+
+    return ( p4_field,
+             p4_type,
+             p4_length,
+             p4_class,
+             p4_default,
+             p4_values,
+             p4_comment,
+             trans,
+             )
 
 def configuration(config):
     # Check Bugzilla specific configuration parameters.
@@ -291,28 +525,12 @@ def configuration(config):
                                types.StringType, 'strings')
     check_config.check_list_of(config, 'replicated_fields',
                                types.StringType, 'strings')
-    if os.name == 'nt':
-        check_config.check_bool(config, 'use_windows_event_log')
-    if os.name == 'posix':
-        check_config.check_bool(config, 'use_system_log')
+    check_config.check_list_of(config, 'omitted_fields',
+                               types.StringType, 'strings')
+    check_config.check_list_of_string_pairs(config, 'field_names')
 
-    if config.bugzilla_directory != None:
-        # strip any trailing / character
-        if (len(config.bugzilla_directory) > 1
-            and config.bugzilla_directory[-1:] == '/'):
-            config.bugzilla_directory = config.bugzilla_directory[:-1]
-        if not os.path.isdir(config.bugzilla_directory):
-            # "Configuration parameter 'bugzilla_directory' does not
-            # name a directory."
-            raise error, catalog.msg(303)
-        if os.name == 'posix':
-            processmail = config.bugzilla_directory + '/processmail'
-        elif os.name == 'nt':
-            processmail = config.bugzilla_directory + '\\processmail.pl'
-        if not os.access(processmail, os.X_OK):
-            # "Configuration parameter 'bugzilla_directory' does not
-            # name a directory containing a processmail script."
-            raise error, catalog.msg(304)
+    check_bugzilla_directory(config)
+    check_field_lists(config)
 
     # Handle logger.  We need a list of logger objects:
     log_params = {
@@ -341,7 +559,6 @@ def configuration(config):
                              (),
                              log_params))
 
-
     # now a single logger object which logs to all of the logger objects
     # in our list:
     config.logger = logger.multi_logger(loggers)
@@ -360,46 +577,11 @@ def configuration(config):
     # Get the types of the 'bugs' table from Bugzilla
     bugs_types = config.bugzilla.get_types('bugs')
 
-    # Get the types of the 'profiles' table from Bugzilla.  In
-    # particular we need to know the size of the 'login_name' column.
-    profiles_types = config.bugzilla.get_types('profiles')
-    if not profiles_types.has_key('login_name'):
-        # "Bugzilla's table 'profiles' does not have a 'login_name'
-        # column."
-        raise error, catalog.msg(305)
-    if profiles_types['login_name']['type'] != 'text':
-        # "The 'login_name' column of Bugzilla's 'profiles' table does
-        # not have a 'text' type."
-        raise error, catalog.msg(306)
-    user_name_length = profiles_types['login_name']['length']
+    # Check field names against Bugzilla database and construct
+    # a map, Bugzilla field name to Perforce field name.
+    fields_bz_to_p4 = get_fields_bz_to_p4(config, bugs_types)
 
-    required_columns = ['bug_status', 'resolution', 'assigned_to',
-                        'short_desc']
-    for column in required_columns:
-        if not bugs_types.has_key(column):
-            # "Bugzilla's table 'bugs' does not have a '%s' column."
-            raise error, catalog.msg(307, column)
-
-    if not bugs_types['bug_status']['type'] == 'enum':
-        # "The 'bug_status' column of Bugzilla's 'bugs' table is not an
-        # enum type."
-        raise error, catalog.msg(308)
-    if not bugs_types['resolution']['type'] == 'enum':
-        # "The 'resolution' column of Bugzilla's 'bugs' table is not an
-        # enum type."
-        raise error, catalog.msg(309)
-    if not 'FIXED' in bugs_types['resolution']['values']:
-        # "The 'resolution' column of Bugzilla's 'bugs' table does not
-        # have a 'FIXED' value."
-        raise error, catalog.msg(310)
-
-    # Make a list of (Bugzilla state, Perforce state) pairs.
-    state_pairs = make_state_pairs(bugs_types['bug_status']['values'],
-                                   config.closed_state)
-
-    # Make a list of possible resolutions.
-    (resolutions,default_resolution) = translate_enum(bugs_types,
-                                                      'resolution')
+    user_name_length = get_user_name_length(config)
 
     # strict user translator doesn't allow unknown users
     strict_user_translator = dt_bugzilla.user_translator(
@@ -409,55 +591,23 @@ def configuration(config):
     lax_user_translator = dt_bugzilla.user_translator(
         config.replicator_address, config.p4_user, allow_unknown = 1)
 
-    # Work out the legal values of the State field in the jobspec.  Note
-    # that "closed" must be a legal state because "p4 fix -c CHANGE
-    # JOBNAME" always sets the State to "closed" even if "closed" is not
-    # a legal value.  See job000225.
-    legal_states = map((lambda x: x[1]), state_pairs)
-    if 'closed' not in legal_states:
-        legal_states.append('closed')
-    state_values = string.join(legal_states, '/')
-
     # p4_fields maps Bugzilla field name to the jobspec data (number,
     # name, type, length, dispositon, preset, values, help text,
-    # translator).  The five fields 101 to 105 are predefined because
-    # they are needed by both Bugzilla and Perforce and the replicator.
-    # The fields Job and Date are special: they are not replicated from
-    # Bugzilla but are required by Perforce, so we have them here.
-    # Note that their help text is given (the other help texts will be
-    # obtained from the bz_field_map).
+    # translator).  The fields Job and Date are special: they are not
+    # replicated from Bugzilla but are required by Perforce, so we
+    # have them here.  Note that their help text is given (the other
+    # help texts will be obtained from the bz_field_map).
+
     p4_fields = { \
         '(JOB)':       ( 101, 'Job', 'word', 32, 'required',
                          None, None,
                          "The job name.",
                          None ),
-        'bug_status':  ( 102, bz_field_map['bug_status'][0], 'select',
-                         bugs_types['bug_status']['length'],
-                         'required',
-                         state_pairs[0][1],
-                         state_values,
-                         bz_field_map['bug_status'][1],
-                         dt_bugzilla.status_translator(state_pairs) ),
-        'assigned_to': ( 103, bz_field_map['assigned_to'][0], 'word',
-                         user_name_length, 'required',
-                         '$user', None,
-                         bz_field_map['assigned_to'][1],
-                         strict_user_translator),
         '(DATE)':      ( 104, 'Date', 'date', 20, 'always',
                          '$now', None,
                          "The date this job was last modified.",
                          None ),
-        'short_desc':  ( 105, bz_field_map['short_desc'][0], 'text',
-                         bugs_types['short_desc']['length'], 'required',
-                         '$blank', None,
-                         bz_field_map['short_desc'][1],
-                         dt_bugzilla.text_translator() ),
-        'resolution':  ( 106, bz_field_map['resolution'][0], 'select',
-                         bugs_types['resolution']['length'], 'required',
-                         default_resolution,
-                         resolutions,
-                         bz_field_map['resolution'][1],
-                         enum_translator),
+        # P4DTI fields:
         '(FILESPECS)': ( 191, 'P4DTI-filespecs', 'text', 0, 'optional',
                          None, None,
                          "Associated filespecs.",
@@ -478,115 +628,89 @@ def configuration(config):
                          None ),
         }
 
-    # The other replicated fields will be sequential from this field id.
+    if fields_bz_to_p4.has_key('bug_status'):
+        if bugs_types['bug_status']['type'] != 'enum':
+            # "The 'bug_status' column of Bugzilla's 'bugs' table is not an
+            # enum type."
+            raise error, catalog.msg(308)
+        # Make a list of (Bugzilla state, Perforce state) pairs.
+        state_pairs = make_state_pairs(bugs_types['bug_status']['values'],
+                                       config.closed_state)
+
+
+        # Work out the legal values of the State field in the jobspec.  Note
+        # that "closed" must be a legal state because "p4 fix -c CHANGE
+        # JOBNAME" always sets the State to "closed" even if "closed" is not
+        # a legal value.  See job000225.
+        legal_states = map((lambda x: x[1]), state_pairs)
+        if 'closed' not in legal_states:
+            legal_states.append('closed')
+        state_values = string.join(legal_states, '/')
+        p4_fields['bug_status'] = ( 102,
+                                    fields_bz_to_p4['bug_status'],
+                                    'select',
+                                    bugs_types['bug_status']['length'],
+                                    'required',
+                                    state_pairs[0][1],
+                                    state_values,
+                                    bz_field_map['bug_status'][1],
+                                    dt_bugzilla.status_translator(state_pairs))
+
+    if fields_bz_to_p4.has_key('assigned_to'):
+        p4_fields['assigned_to'] = ( 103,
+                                     fields_bz_to_p4['assigned_to'],
+                                     'word', user_name_length,
+                                     'required',
+                                     '$user', None,
+                                     bz_field_map['assigned_to'][1],
+                                     strict_user_translator)
+
+    if fields_bz_to_p4.has_key('short_desc'):
+        p4_fields['short_desc'] = ( 105,
+                                    fields_bz_to_p4['short_desc'],
+                                    'text',
+                                    bugs_types['short_desc']['length'],
+                                    'required',
+                                    '$blank', None,
+                                    bz_field_map['short_desc'][1],
+                                    dt_bugzilla.text_translator() )
+
+    if fields_bz_to_p4.has_key('resolution'):
+        if bugs_types['resolution']['type'] != 'enum':
+            # "The 'resolution' column of Bugzilla's 'bugs' table is not an
+            # enum type."
+            raise error, catalog.msg(309)
+        if not 'FIXED' in bugs_types['resolution']['values']:
+            # "The 'resolution' column of Bugzilla's 'bugs' table does not
+            # have a 'FIXED' value."
+            raise error, catalog.msg(310)
+
+        # Make a list of possible resolutions.
+        (resolutions,
+         default_resolution) = translate_enum('resolution',
+                                              bugs_types['resolution'])
+        p4_fields['resolution'] = ( 106,
+                                    fields_bz_to_p4['resolution'],
+                                    'select',
+                                    bugs_types['resolution']['length'],
+                                    'required',
+                                    default_resolution,
+                                    resolutions,
+                                    bz_field_map['resolution'][1],
+                                    enum_translator)
+
+    # Additional replicated fields will be sequential from this field id.
     p4_field_id = 110
 
     # Go through the replicated_fields list, build structures and add
     # them to p4_fields.
-    for dt_field_name in config.replicated_fields:
-        # Field must not be already in the p4_fields structure.
-        if p4_fields.has_key(dt_field_name):
-            if dt_field_name in ['bug_status', 'assigned_to',
-                                 'short_desc', 'resolution']:
-                # "Field '%s' specified in 'replicated_fields' is a
-                # system field: leave it out!"
-                raise error, catalog.msg(311, dt_field_name)
-            else:
-                # "Field '%s' appears twice in 'replicated_fields'."
-                raise error, catalog.msg(312, dt_field_name)
-
-        # Field must exist in Bugzilla
-        if not bugs_types.has_key(dt_field_name):
-            # "Field '%s' specified in 'replicated_fields' list not in
-            # Bugzilla 'bugs' table."
-            raise error, catalog.msg(313, dt_field_name)
-
-        type = bugs_types[dt_field_name]
-        dt_field_type = type['type']
-        p4_field_values = None
-        p4_field_length = None
-
-        if bz_field_map.has_key(dt_field_name):
-            p4_field_name = bz_field_map[dt_field_name][0]
-            p4_field_comment = bz_field_map[dt_field_name][1]
-        else:
-            p4_field_name = "bugzilla_" + dt_field_name
-            p4_field_comment = None
-
-        if p4_field_comment == None:
-            p4_field_comment = ("Bugzilla's '%s' field" % dt_field_name)
-
-        # if there is a default, use it.
-        if type.get('default'):
-            p4_field_class = 'default'
-            p4_field_default = type['default']
-        else:
-            p4_field_class = 'optional'
-            p4_field_default = None
-
-        if dt_field_type == 'other':
-            # "Field '%s' specified in 'replicated_fields' list has type
-            # '%s': this is not yet supported by P4DTI."
-            raise error, catalog.msg(314, (dt_field_name,
-                                           type['sql_type']))
-        elif dt_field_type == 'float':
-            # "Field '%s' specified in 'replicated_fields' list has
-            # floating-point type: this is not yet supported by P4DTI."
-            raise error, catalog.msg(315, dt_field_name)
-        elif dt_field_type == 'user':
-            p4_field_type = 'word'
-            p4_field_length = user_name_length
-            trans = strict_user_translator
-        elif dt_field_type == 'enum':
-            p4_field_type = 'select'
-            p4_field_values, p4_field_default = translate_enum(
-                bugs_types, dt_field_name)
-            trans = enum_translator
-        elif dt_field_type == 'int':
-            p4_field_type = 'word'
-            trans = dt_bugzilla.int_translator()
-        elif dt_field_type == 'date':
-            p4_field_type = 'date'
-            p4_field_length = 20
-            trans = dt_bugzilla.date_translator()
-        elif dt_field_type == 'timestamp':
-            p4_field_type = 'date'
-            p4_field_length = 20
-            trans = dt_bugzilla.timestamp_translator()
-        elif dt_field_type == 'text':
-            p4_field_type = 'text'
-            trans = dt_bugzilla.text_translator()
-
-        # "p4 -G" uses the field "code" to indicate whether the Perforce
-        # command succeeded or failed.  See job000003.
-        if p4_field_name == 'code':
-            # "You can't have a field called 'code' in the Perforce
-            # jobspec."
-            raise error, catalog.msg(316)
-
-        # Fixed-length fields get the length from Bugzilla.
-        if p4_field_length == None:
-            p4_field_length = type['length']
-
-        if dt_field_name in read_only_fields:
-            p4_field_comment = (p4_field_comment +
-                                "  DO NOT MODIFY.")
-
-        if dt_field_name in append_only_fields:
-            p4_field_comment = (p4_field_comment +
-                                "  ONLY MODIFY BY APPENDING.")
-
-        p4_fields[dt_field_name] = \
-            ( p4_field_id,
-              p4_field_name,
-              p4_field_type,
-              p4_field_length,
-              p4_field_class,
-              p4_field_default,
-              p4_field_values,
-              p4_field_comment,
-              trans
-              )
+    for bz_field in config.replicated_fields:
+        p4_field = fields_bz_to_p4[bz_field]
+        p4_fields[bz_field] = ((p4_field_id, ) +
+                               make_p4_field_spec(bz_field, p4_field,
+                                                  bugs_types[bz_field],
+                                                  user_name_length,
+                                                  strict_user_translator))
         p4_field_id = p4_field_id + 1
         if p4_field_id >= 191:
             # "Too many fields to replicate: Perforce jobs can contain
@@ -606,8 +730,8 @@ def configuration(config):
 
     # Set configuration parameters needed by the replicator.
     config.date_translator = dt_bugzilla.date_translator()
-    config.job_owner_field = 'Assigned_To'
-    config.job_status_field = 'Status'
+    config.job_owner_field = fields_bz_to_p4.get('assigned_to', 'User')
+    config.job_status_field = fields_bz_to_p4.get('bug_status', 'Status')
     config.job_date_field = 'Date'
     config.jobspec = jobspec
     config.prepare_issue_advanced = prepare_issue_advanced
@@ -630,11 +754,11 @@ def configuration(config):
 #
 # [GDR 2001-11-14] "Perforce Defect Tracking Integration Advanced
 # Administrator's Guide"; Gareth Rees; Ravenbrook Limited; 2001-11-14;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/manual/aag/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/manual/aag/>.
 #
 # [GDR 2000-10-16] "Perforce Defect Tracking Integration Integrator's
 # Guide"; Gareth Rees; Ravenbrook Limited; 2000-10-16;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/manual/ig/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/manual/ig/>.
 #
 #
 # B. DOCUMENT HISTORY
@@ -746,8 +870,14 @@ def configuration(config):
 #
 # 2003-05-21 NB Fixed some comments which incorrectly referred to TeamTrack.
 #
+# 2003-11-25 NB Rewrite the jobspec-creation code, and the handling of
+# the relevant configuration parameters.  This is a step towards
+# configurable jobspec.
 #
-# C. COPYRIGHT AND LICENCE
+# 2004-05-28 NB Add bugmail as alternative to processmail.
+#
+#
+# C. COPYRIGHT AND LICENSE
 #
 # This file is copyright (c) 2001 Perforce Software, Inc.  All rights
 # reserved.
@@ -778,4 +908,4 @@ def configuration(config):
 # DAMAGE.
 #
 #
-# $Id: //info.ravenbrook.com/project/p4dti/version/2.0/code/replicator/configure_bugzilla.py#2 $
+# $Id: //info.ravenbrook.com/project/p4dti/version/2.3/code/replicator/configure_bugzilla.py#2 $
