@@ -76,7 +76,7 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
                     'qa_contact',
                     'everconfirmed',
                     'assigned_to',
-                    'groupset',
+                    'groups',
                     'bug_status',
                     'resolution']:
             assert bug.has_key(key)
@@ -157,8 +157,12 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
         self.make_p4dti_bug(jobname, created=0)
 
     # Check Bugzilla permissions.
-    # In Bugzilla 2.10 to 2.16.3, permissions are checked in
-    # CheckCanChangeField() in process_bug.cgi.  This is the test:
+    # 
+    # In Bugzilla, permissions are mostly checked in
+    # CheckCanChangeField() in process_bug.cgi.  The test changed in
+    # Bugzilla 2.17.
+    # 
+    # This was the test in Bugzilla versions up to 2.16.5:
     #
     # 0. disabled users can't make any change;
     #    (this is not in CheckCanChangeField():
@@ -166,9 +170,9 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
     # 1. anyone can make a null change;
     # 2. anyone can make a change which just adds or removes
     #    whitespace at the beginning or end of a value;
-    # 3. anyone can add a description record;
+    # 3. anyone can add a comment record;
     # 4. anyone in the "editbugs" group can make any change;
-    # 5. the bug's reporter can always change the bug status;
+    # 5. the bug's reporter can change the bug status;
     # 6. anyone in the "canconfirm" group can change the status to any
     #    opened status.
     # 7. anyone can change the status to any opened status if the bug has
@@ -179,16 +183,37 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
     #
     # An opened status is NEW, REOPENED, or ASSIGNED.
     #
+    # This is the test in Bugzilla versions from 2.17.7:
+    #
+    # 0. disabled users can't make any change;
+    #    (this is not in CheckCanChangeField():
+    #    disabled users can't even log in)
+    # 1. anyone can make a null change;
+    # 2. anyone can make a change which just adds or removes
+    #    whitespace at the beginning or end of a value;
+    # 3. anyone can add a comment record;
+    # 4. anyone in the "editbugs" group can make any change;
+    # 10. anyone in the "canconfirm" group can change the status from
+    #    UNCONFIRMED to any opened status;
+    # 11. The assigned_to or qa_contact can make any change;
+    # 12. The reporter can make any change except:
+    #    12.1. changing the status from UNCONFIRMED to any opened state;
+    #    12.2. changing the target milestone;
+    #    12.3. changing the priority (unless the
+    #         letsubmitterchoosepriority parameter is set)
+    # 9. Nobody else can make a change.
+    # 
     # Note that there is not a check made of whether the user is in
-    # the bug group of the bug.  There is an implicit check of this in
-    # buglist.pl and bug_form.pl; if the user is not in the bug group,
-    # the bug is not displayed.
+    # the bug group(s) of the bug.  There is an implicit check of this
+    # in buglist.pl and bug_form.pl; if the user is not in the bug
+    # group(s), the bug is not displayed and cannot be changed.
 
     def opened_status(self, status):
         assert isinstance(status, types.StringType)
         return status in ['NEW', 'REOPENED', 'ASSIGNED']
 
-    def can_change_field(self, user, canconfirm, key, old, new):
+    def can_change_field(self, user, user_groups, key, new, pre_217):
+        old = self.bug[key]
         assert isinstance(key, types.StringType)
         assert type(old) == type(new)
         # 0. disabled users handled by check_permissions().
@@ -203,23 +228,49 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
         if key == 'longdesc':
             return 1
         # 4. editbugs handled by check_permissions().
-        # 5, 6, 7:
-        if key == 'bug_status':
-            # 5: reporter can change status
-            if user == self.bug['reporter']:
+        if pre_217:
+            if key == 'bug_status':
+                # 5: reporter can change status
+                if user == self.bug['reporter']:
+                    return 1
+                if self.opened_status(new):
+                    # 6. canconfirm
+                    if 'canconfirm' in user_groups:
+                        return 1
+                    # 7. everconfirmed
+                    if self.bug['everconfirmed'] == 1:
+                        return 1
+            if (key != 'bug_status') or not self.opened_status(new):
+                # 8. reporter/assigned_to/qa_contact
+                if (user == self.bug['reporter']
+                    or user == self.bug['assigned_to']
+                    or user == self.bug['qa_contact']):
+                    return 1
+        else:
+            # 10: canconfirm can confirm
+            if ((key == 'bug_status') and
+                (old == 'UNCONFIRMED') and
+                self.opened_status(new) and
+                ('canconfirm' in user_groups)):
                 return 1
-            if self.opened_status(new):
-                # 6. canconfirm
-                if canconfirm:
-                    return 1
-                # 7. everconfirmed
-                if self.bug['everconfirmed'] == 1:
-                    return 1
-        if (key != 'bug_status') or not self.opened_status(new):
-            # 8. reporter/assigned_to/qa_contact
-            if (user == self.bug['reporter']
-                or user == self.bug['assigned_to']
-                or user == self.bug['qa_contact']):
+            # 11. assigned_to or qa_contact can make any change
+            if ((user == self.bug['assigned_to']) or
+                (user == self.bug['qa_contact'])):
+                return 1
+            # 12. reporter can do anything except:
+            if user == self.bug['reporter']:
+                # 12.1. confirming 
+                if ((key == 'bug_status') and
+                    (old == 'UNCONFIRMED') and
+                    self.opened_status(new)):
+                    return 0
+                # 12.2 changing target milestone
+                if key == 'target_milestone':
+                    return 0
+                # 12.3 changing priority
+                if ((key == 'priority') and
+                    not int(self.dt.params.get('letsubmitterchoosepriority', '1'))):
+                    return 0
                 return 1
         # 9. nobody else
         return 0
@@ -235,28 +286,29 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
             # "User %d is disabled, so cannot edit bug %d."
             raise error, catalog.msg(537, (user, self.bug['bug_id']))
 
-        user_groupset = self.dt.user_groupset(user)
-        bug_groupset = self.bug['groupset']
-
-        assert (bug_groupset == 0
-                or self.dt.singleton_groupset(bug_groupset))
+        bug_groups = self.bug['groups']
+        user_groups = self.dt.bugzilla.user_groups(user)
+        product_groups = self.dt.bugzilla.product_editor_groups(self.bug['product'])
+        pre_217 = self.dt.bugzilla.bugzilla_version < '2.17'
 
         # 4. user in editbugs can make any change.
-        if self.dt.groupset_has_named_group(user_groupset, 'editbugs'):
+        if 'editbugs' in user_groups:
             return
-        # Are we in the bug's groupset?
-        # Bugzilla doesn't check this, but there is an implicit
-        # check because Bugzilla won't show this bug to this user.
-        if ((bug_groupset != 0)
-            and (user_groupset & bug_groupset) != bug_groupset):
-            # "User %d isn't in the right bug group to edit bug %d."
-            raise error, catalog.msg(500, (user, self.bug['bug_id']))
 
-        canconfirm = self.dt.groupset_has_named_group(user_groupset,
-                                                      'canconfirm')
+        # Are we in the bug's groups?
+        for g in bug_groups:
+            if g not in user_groups:
+                # "User %d must be in group '%s' to edit bug %d."
+                raise error, catalog.msg(555, (user, g, self.bug['bug_id']))
+
+        for g in product_groups:
+            if g not in user_groups:
+                # "User %d must be in group '%s' to edit bug %d in product '%s'."
+                raise error, catalog.msg(556, (user, g, self.bug['bug_id'],
+                                               self.bug['product']))
+
         for key, newvalue in changes.items():
-            if not self.can_change_field(user, canconfirm,
-                                         key, self.bug[key], newvalue):
+            if not self.can_change_field(user, user_groups, key, newvalue, pre_217):
                 # "User %d doesn't have permission to change field '%s'
                 # of bug %d to %s."
                 raise error, catalog.msg(501, (user, key,
@@ -296,7 +348,6 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
         return status in ['RESOLVED', 'VERIFIED', 'CLOSED']
 
     def enforce_invariants(self, changes):
-
         if (changes.has_key('resolution')
             and changes['resolution'] == 'DUPLICATE'):
             # "The P4DTI does not support marking bugs as DUPLICATE from
@@ -304,7 +355,6 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
             raise error, catalog.msg(502)
 
         if changes.has_key('bug_status'):
-
             # We are making a transition.
             if not self.transition_allowed(self.bug['bug_status'],
                                            changes['bug_status']):
@@ -401,7 +451,7 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
         self.dt.bugzilla.update_bug(changes_bug, self.bug, user)
 
         # Add processmail script to pending queue.
-        self.dt.bugzilla.processmail(self.bug['bug_id'], user)
+        self.dt.bugzilla.bugmail(self.bug['bug_id'], user)
 
         # Now the bug is updated in the database, update our copy.
         for key, value in changes_bug.items():
@@ -415,12 +465,7 @@ class bugzilla_bug(dt_interface.defect_tracker_issue):
 
     # Delete this bug.
     def delete(self):
-        bug_id = self.bug['bug_id']
-        self.dt.bugzilla.delete_fixes_for_bug(bug_id)
-        self.dt.bugzilla.delete_filespecs_for_bug(bug_id)
-        if self.p4dti_bug:
-            self.dt.bugzilla.delete_p4dti_bug(bug_id)
-        self.dt.bugzilla.delete_bug(bug_id)
+        self.dt.bugzilla.delete_bug(self.bug['bug_id'])
 
 
 # 4. BUGZILLA FIX INTERFACE
@@ -531,12 +576,14 @@ class dt_bugzilla(dt_interface.defect_tracker):
     rid = None
     sid = None
     bugzilla = None
+    cached_users = 0 # Are the user records fresh?
 
     def __init__(self, config):
         self.config = config
         self.rid = config.rid
         self.sid = config.sid
 	self.bugzilla = config.bugzilla
+        self.cached_users = 0
 
     def log(self, msg, args = ()):
         if not isinstance(msg, message.message):
@@ -549,11 +596,12 @@ class dt_bugzilla(dt_interface.defect_tracker):
 
     def poll_start(self):
         self.bugzilla.lock_tables()
-        self.bugzilla.clear_processmails()
+        self.cached_users = 0
+        self.bugzilla.clear_caches()
 
     def poll_end(self):
         self.bugzilla.unlock_tables()
-        self.bugzilla.invoke_processmails()
+        self.bugzilla.invoke_deferred_commands()
 
     def changed_entities(self):
         replication = self.bugzilla.new_replication()
@@ -600,16 +648,12 @@ class dt_bugzilla(dt_interface.defect_tracker):
             dict['login_name'] = email
             dict['password'] = self.config.migrated_user_password
             dict['realname'] = fullname
-            groupset = 0L
-            self.init_group_tables()
-            for group in self.config.migrated_user_groups:
-                if self.group_by_name.has_key(group):
-                    groupset = (groupset
-                                + self.group_by_name[group]['bit'])
-                else:
+            all_groups = self.bugzilla.groups()
+            for g in self.config.migrated_user_groups:
+                if not all_groups.has_key(g):
                     # "'%s' not a Bugzilla group."
                     raise error, catalog.msg(535, group)
-            dict['groupset'] = groupset
+            dict['groups'] = self.config.migrated_user_groups
             userid = self.bugzilla.add_user(dict)
             # "Perforce user '%s <%s>' added to Bugzilla as user %d."
             self.log(534, (p4user, email, userid))
@@ -628,7 +672,7 @@ class dt_bugzilla(dt_interface.defect_tracker):
             dict['login_name'] = email
             dict['password'] = self.config.migrated_user_password
             dict['realname'] = "Perforce defect tracking integration"
-            dict['groupset'] = 0L
+            dict['groups'] = []
             dict['disabledtext'] = "This user can access Bugzilla only as the P4DTI replicator process"
             userid = self.bugzilla.add_user(dict)
             # "Perforce replicator user <%s> added to Bugzilla as user %d."
@@ -637,11 +681,12 @@ class dt_bugzilla(dt_interface.defect_tracker):
 
     def new_issues_start(self):
         self.bugzilla.lock_tables()
-        self.bugzilla.clear_processmails()
+        self.cached_users = 0
+        self.bugzilla.clear_caches()
 
     def new_issues_end(self):
         self.bugzilla.unlock_tables()
-        self.bugzilla.invoke_processmails()
+        self.bugzilla.invoke_deferred_commands()
 
     # new_issue_defaults(dict).  Supply default values for the product,
     # component and version fields where possible (see [AG, 6.2]).
@@ -677,7 +722,7 @@ class dt_bugzilla(dt_interface.defect_tracker):
             if (value != '' and
                 value != 0 and
                 not key in ['resolution',
-                           'groupset',
+                           'groups',
                            'assigned_to',
                            'bug_file_loc',
                            'bug_severity',
@@ -772,26 +817,33 @@ class dt_bugzilla(dt_interface.defect_tracker):
         email = self.bugzilla.email_from_userid(user)
 
         # permissions for the reporter to create the bug.
-        user_groupset = self.user_groupset(user)
-        product_group = self.bugzilla.product_group(product_name)
-        if product_group != None:
-            if (user_groupset & product_group['bit']) == 0:
-                # "User '%s' isn't in Bugzilla product group for product
-                # '%s'; migrating bug anyway."
-                self.log(527, (email, product_name))
+        user_groups = self.bugzilla.user_groups(user)
+        product_groups = self.bugzilla.product_creator_groups(product_name)
+        for group in product_groups:
+            if group not in user_groups:
+                # "User '%s' isn't in Bugzilla group '%s' required to make bug
+                # for product '%s'; creating bug anyway."
+                self.log(527, (email, group, product_name))
 
-        # if groupset, check valid otherwise set to default (0)
-        if dict.has_key('groupset'):
-            groupset = dict['groupset']
-            if (groupset & self.valid_groupset_bits()) != groupset:
-                # "Can't create Bugzilla bug with invalid groupset
+        if not dict.has_key('groups'):
+            dict['groups'] = []
+
+        bug_groups = dict['groups']
+
+        all_groups = self.bugzilla.groups()
+        for group in bug_groups:
+            if not all_groups.has_key(group):
+                # "Can't create Bugzilla bug with invalid group
                 # '%s'."
-                raise error, catalog.msg(528, groupset)
-        else:
-            if product_group != None:
-                dict['groupset'] = product_group['bit']
-            else:
-                dict['groupset'] = 0
+                raise error, catalog.msg(528, group)
+
+        # what groups must a bug go into?
+        mandatory_groups = self.bugzilla.new_bug_groups(product_name, user_groups)
+        for group in mandatory_groups:
+            if not group in bug_groups:
+                bug_groups.append(group)
+
+        dict['groups'] = bug_groups
 
         if product['votestoconfirm'] != 0:
             default_bug_status = 'UNCONFIRMED'
@@ -803,12 +855,10 @@ class dt_bugzilla(dt_interface.defect_tracker):
             # if the bug_status is not the default for this product,
             # check the user has the permissions.
             if (bug_status != default_bug_status
-                and not self.groupset_has_named_group(user_groupset,
-                                                      "editbugs")
-                and not self.groupset_has_named_group(user_groupset,
-                                                      "canconfirm")):
+                and not ('editbugs' in user_groups)
+                and not ('canconfirm' in user_groups)):
                 # "User '%s' doesn't have permissions to create Bugzilla
-                # bug for product '%s' with status '%s'; migrating bug
+                # bug for product '%s' with status '%s'; creating bug
                 # anyway."
                 self.log(529, (email, product_name, bug_status))
         else:
@@ -857,7 +907,7 @@ class dt_bugzilla(dt_interface.defect_tracker):
         if not dict.has_key('longdesc'):
             dict['longdesc'] = 'No initial comment.'
 
-        bug_id = self.bugzilla.add_bug(dict, email)
+        bug_id = self.bugzilla.add_bug(dict)
         bug = self.issue(bug_id)
         # in future might want another jobname here.
         bug.make_p4dti_bug(jobname, created=1)
@@ -893,7 +943,8 @@ class dt_bugzilla(dt_interface.defect_tracker):
         changes['client'] = client
         changes['p4date'] = date
         changes['description'] = description
-        changes['flags'] = (status == 'submitted')
+        # "(x and 1) or 0" gives the int value of the boolean x.
+        changes['flags'] = ((status == 'submitted') and 1) or 0
         changes['user'] = user
         for key, value in changes.items():
             if (not dt_changelist.has_key(key)
@@ -903,43 +954,6 @@ class dt_bugzilla(dt_interface.defect_tracker):
                 del changes[key]
         return changes
 
-    # groups.
-    group_by_name = {}
-    group_by_bit = {}
-
-    def init_group_tables(self):
-        groups = self.bugzilla.groups()
-        for group in groups:
-            self.group_by_name[group['name']] = group
-            self.group_by_bit[group['bit']] = group
-
-    def groupset_has_named_group(self, groupset, group):
-        if not self.group_by_name.has_key(group):
-            self.init_group_tables()
-        if not self.group_by_name.has_key(group):
-            # "Bugzilla does not have a group called '%s'."
-            raise error, catalog.msg(507, group)
-        groupbit = self.group_by_name[group]['bit']
-        return (groupset & groupbit) == groupbit
-
-    def valid_groupset_bits(self):
-        valid = 0L
-        if not self.group_by_bit:
-            self.init_group_tables()
-        for bit in self.group_by_bit.keys():
-            valid = valid + bit
-        return valid
-
-    def user_has_named_group(self, user, group):
-        groupset = self.bugzilla.groupset_from_userid(user)
-        return self.groupset_has_named_group(groupset, group)
-
-    def user_groupset(self, user):
-        return self.bugzilla.groupset_from_userid(user)
-
-    def singleton_groupset(self, groupset):
-        return groupset != 0 and (groupset & (groupset - 1)) == 0
-
 # 7. TRANSLATORS
 #
 # These classes translate values of particular types between Bugzilla
@@ -1284,6 +1298,8 @@ class user_translator(translator.user_translator):
     # the Bugzilla P4DTI user id.
     allow_unknown = 0
 
+    tables_populated = 0
+
     def __init__(self, bugzilla_user, p4_user,
                  allow_unknown = 0):
         self.bugzilla_user = string.lower(bugzilla_user)
@@ -1294,6 +1310,9 @@ class user_translator(translator.user_translator):
     # Perforce username.
     def init_users(self, bz, p4):
 
+        if self.tables_populated and bz.cached_users:
+            return
+
         # Clear the maps.
         self.user_bz_to_p4 = {}
         self.user_p4_to_bz = {}
@@ -1462,6 +1481,8 @@ class user_translator(translator.user_translator):
         # always translate 0 to 'None' and back again
         self.user_p4_to_bz['None'] = 0
         self.user_bz_to_p4[0] = 'None'
+        self.tables_populated = 1
+        bz.cached_users = 1
 
     def unmatched_users(self, bz, p4):
         self.init_users(bz, p4)
@@ -1504,7 +1525,7 @@ class user_translator(translator.user_translator):
                 raise error, catalog.msg(514, p4_user)
 
     def translate_0_to_1(self, bz_user, bz, p4, issue=None, job=None):
-        if not self.user_bz_to_p4.has_key(bz_user):
+        if not bz.cached_users:
             self.init_users(bz, p4)
         if self.user_bz_to_p4.has_key(bz_user):
             return self.user_bz_to_p4[bz_user]
@@ -1517,19 +1538,19 @@ class user_translator(translator.user_translator):
 #
 # [GDR 2000-10-16] "Perforce Defect Tracking Integration Integrator's
 # Guide"; Gareth Rees; Ravenbrook Limited; 2000-10-16;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/manual/ig/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/manual/ig/>.
 #
 # [NB 2000-11-14a] "Bugzilla database schema"; Nick Barnes; Ravenbrook
 # Limited; 2000-11-14;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/design/bugzilla-schema/>.
+# <http://www.ravenbrook.com/project/p4dti/tool/cgi/bugzilla-schema/>.
 #
 # [NB 2000-11-14b] "Bugzilla database schema extensions for integration
 # with Perforce"; Nick Barnes; Ravenbrook Limited; 2000-11-14;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/design/bugzilla-p4dti-schema/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/design/bugzilla-p4dti-schema/>.
 #
 # [NB 2000-11-14c] "Python interface to Bugzilla: design"; Nick Barnes;
 # Ravenbrook Limited; 2000-11-14;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/design/python-bugzilla-interface/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/design/python-bugzilla-interface/>.
 #
 #
 # B. DOCUMENT HISTORY
@@ -1654,10 +1675,15 @@ class user_translator(translator.user_translator):
 #
 # 2003-05-22 NB Add_replicator_user method to add the replicator user.
 # Also update the can_change_field() logic and comments.
-
+#
+# 2004-05-28 NB Bugzilla 2.17.7 support: change processmail invocation
+# system to use bugmail when present; changed interface to bugzilla.py
+# for groups, products, and components. support membership of a single
+# bug in multiple groups; changed bug change permission rules; test
+# per-product group memberships when creating or editing bugs.
 #
 #
-# C. COPYRIGHT AND LICENCE
+# C. COPYRIGHT AND LICENSE
 #
 # This file is copyright (c) 2001 Perforce Software, Inc.  All rights
 # reserved.
@@ -1688,4 +1714,4 @@ class user_translator(translator.user_translator):
 # DAMAGE.
 #
 #
-# $Id: //info.ravenbrook.com/project/p4dti/version/2.0/code/replicator/dt_bugzilla.py#2 $
+# $Id: //info.ravenbrook.com/project/p4dti/version/2.3/code/replicator/dt_bugzilla.py#2 $
