@@ -233,29 +233,32 @@ class replicator:
     def check_first_time(self):
         if (self.config.jobspec
             and not self.p4.jobspec_has_p4dti_fields(
-            self.p4.get_jobspec())
+            self.p4.get_jobspec(),
+            warn = 0)
             and self.p4.run('jobs')):
             # "You must delete your Perforce jobs before running the
             # P4DTI for the first time.  See section 5.2.3 of the
             # Administrator's Guide."
             raise self.error, catalog.msg(914)
 
-    # update_jobspec().  Update the Perforce jobspec with the jobspec
-    # supplied in the replicator configuration.  But only if there is
-    # one in the configuration.  This allows people to make advanced
-    # configurations that use an existing Perforce jobspec [IG, 8.6].
+    # update_and_check_jobspec().  If keep_jobspec is set, check the
+    # current jobspec against the one we want to install.  Otherwise,
+    # just go ahead and install the jobspec.  Advanced configurations
+    # can turn this off by clearing config.jobspec [IG, 8.6].
+    # 
+    # Subsequently, the installed jobspec is checked to ensure that we
+    #can run the P4DTI with it.
 
-    def update_jobspec(self):
-        if self.config.jobspec:
-            self.p4.install_jobspec(self.config.jobspec)
-
-    # check_jobspec().  Check that P4DTI-* fields are present in the
-    # jobspec.
+    def update_and_check_jobspec(self):
+        if self.config.jobspec and not self.config.keep_jobspec:
+                self.p4.install_jobspec(self.config.jobspec)
+        self.check_jobspec()
 
     def check_jobspec(self):
-        if not self.p4.jobspec_has_p4dti_fields(self.p4.get_jobspec()):
-            # "P4DTI fields not found in Perforce jobspec."
-            raise self.error, catalog.msg(836)
+        self.p4.check_jobspec(self.config.jobspec)
+
+    def extend_jobspec(self, force=0):
+        self.p4.extend_jobspec(self.config.jobspec, force)
 
     # start_logger().  Has the logger been started?  If not, start it.
     # (We must be careful not to set the logger counter to 0 more than
@@ -395,6 +398,7 @@ class replicator:
 
     def job_filespecs(self, job):
         assert isinstance(job, types.DictType)
+        # if no P4DTI-filespecs field, do the right thing:
         job_filespecs = job.get('P4DTI-filespecs', '')
         filespecs = string.split(job_filespecs, '\n')
         # Since Perforce text fields are terminated with a newline, the
@@ -578,7 +582,7 @@ class replicator:
         assert isinstance(user, types.StringType)
         # Even though "p4 user -o foo" doesn't actually create a user,
         # it does fail if the user foo doesn't exist and there are no
-        # spare licences.  So trap that case.  See job000204.
+        # spare licenses.  So trap that case.  See job000204.
         try:
             u = self.p4.run('user -o %s' % user)
         except:
@@ -833,17 +837,15 @@ class replicator:
 
     def poll(self):
         self.check_first_time()
-        self.update_jobspec()
-        self.check_jobspec()
+        self.update_and_check_jobspec()
         self.start_logger()
         self.poll_databases()
 
-    # refresh_perforce_jobs().  Delete all Perforce jobs then replicate
-    # all issues from the defect tracker.
+    # refresh_perforce_jobs().  Replicate all issues from the defect
+    # tracker.  Note: does not delete jobs first.
 
     def refresh_perforce_jobs(self):
-        self.update_jobspec()
-        self.check_jobspec()
+        self.update_and_check_jobspec()
         self.replicate_all_dt_to_p4()
         self.start_logger()
         self.clear_logger()
@@ -883,8 +885,7 @@ class replicator:
 
     def prepare_to_run(self):
         self.check_first_time()
-        self.update_jobspec()
-        self.check_jobspec()
+        self.update_and_check_jobspec()
         self.start_logger()
         self.poll_period = self.config.poll_period
         self.mail_startup_message()
@@ -1902,15 +1903,15 @@ class replicator:
 #
 # [GDR 2000-09-13] "Replicator design"; Gareth Rees; Ravenbrook Limited;
 # 2000-09-13;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/design/replicator/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/design/replicator/>.
 #
 # [GDR 2000-10-16] "Perforce Defect Tracking Integration Integrator's
 # Guide"; Gareth Rees; Ravenbrook Limited; 2000-10-16;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/manual/ig/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/manual/ig/>.
 #
 # [RB 2000-08-10] "Perforce Defect Tracking Integration Architecture";
 # Richard Brooksby; Ravenbrook Limited; 2000-08-10;
-# <http://www.ravenbrook.com/project/p4dti/version/2.0/design/architecture/>.
+# <http://www.ravenbrook.com/project/p4dti/version/2.3/design/architecture/>.
 #
 # [Requirements] "Perforce Defect Tracking Integration Project
 # Requirements"; Gareth Rees; Ravenbrook Limited; 2000-05-24;
@@ -1985,7 +1986,7 @@ class replicator:
 # smtp_server is None.
 #
 # 2001-02-14 GDR user_email_address returns None if the user doesn't
-# exist, even when there are no spare licences (see job000204).
+# exist, even when there are no spare licenses (see job000204).
 #
 # 2001-02-16 NB Added replicate-p configuration parameter.
 #
@@ -2153,8 +2154,13 @@ class replicator:
 #
 # 2003-05-30 NB Work around broken client spec or depot list.
 #
+# 2003-12-05 NB Changed interface to jobspec-checking function.
 #
-# C. COPYRIGHT AND LICENCE
+# 2003-12-12 NB Change jobspec-related functions to expose them to new
+# scripts.
+#
+#
+# C. COPYRIGHT AND LICENSE
 #
 # This file is copyright (c) 2001 Perforce Software, Inc.  All rights
 # reserved.
@@ -2185,4 +2191,4 @@ class replicator:
 # DAMAGE.
 #
 #
-# $Id: //info.ravenbrook.com/project/p4dti/version/2.0/code/replicator/replicator.py#2 $
+# $Id: //info.ravenbrook.com/project/p4dti/version/2.3/code/replicator/replicator.py#2 $
